<h3>Contents</h3>

<ul>
   <li>&nbsp; &nbsp;<a href="{{ pathto('installation') }}"> Installation</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('overview') }}">     Overview</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('interactive') }}">   Interactive Displays</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('plotpanel') }}">    PlotPanel API</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('imagepanel') }}"> Image API</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('examples') }}">    Examples</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('other') }}"> Speciality Displays</a></li>
</ul>
<p>


<h3>Downloads</h3>
<p>Current version: <b>{{ release }}</b></p>
<p>Downloads:
<ul>
  <li>&nbsp; &nbsp; <a href="http://pypi.python.org/pypi/wxmplot/">PyPI (Python.org)</a> <p>
       try <tt>pip install wxmplot</tt>
</ul>
<p>

Development version: <br> &nbsp; &nbsp; <a href="https://github.com/newville/wxmplot/">github.com</a> <br>

<hr>
<p>
